"use strict";

require("core-js/modules/es.symbol.description");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildStaticStandalone = buildStaticStandalone;
exports.buildStatic = buildStatic;

var _cpy = _interopRequireDefault(require("cpy"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _webpack = _interopRequireDefault(require("webpack"));

var _shelljs = _interopRequireDefault(require("shelljs"));

var _nodeLogger = require("@storybook/node-logger");

var _cli = require("./cli");

var _config = _interopRequireDefault(require("./config"));

var _managerConfig = _interopRequireDefault(require("./manager/manager-config"));

var _logConfig = require("./logConfig");

var _prebuiltManager = require("./utils/prebuilt-manager");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

async function compileManager(managerConfig, managerStartTime) {
  _nodeLogger.logger.info('=> Compiling manager..');

  return new Promise(function (resolve, reject) {
    (0, _webpack.default)(managerConfig).run(function (error, stats) {
      if (error || !stats || stats.hasErrors()) {
        _nodeLogger.logger.error('=> Failed to build the manager');

        if (error) {
          _nodeLogger.logger.error(error.message);
        }

        if (stats && (stats.hasErrors() || stats.hasWarnings())) {
          var _stats$toJson = stats.toJson(managerConfig.stats),
              warnings = _stats$toJson.warnings,
              errors = _stats$toJson.errors;

          errors.forEach(function (e) {
            return _nodeLogger.logger.error(e);
          });
          warnings.forEach(function (e) {
            return _nodeLogger.logger.error(e);
          });
        }

        process.exitCode = 1;
        reject(error || stats);
        return;
      }

      _nodeLogger.logger.trace({
        message: '=> Manager built',
        time: process.hrtime(managerStartTime)
      });

      stats.toJson(managerConfig.stats).warnings.forEach(function (e) {
        return _nodeLogger.logger.warn(e);
      });
      resolve(stats);
    });
  });
}

async function watchPreview(previewConfig) {
  _nodeLogger.logger.info('=> Compiling preview in watch mode..');

  return new Promise(function () {
    (0, _webpack.default)(previewConfig).watch({
      aggregateTimeout: 1
    }, function (error, stats) {
      if (!error) {
        var statsConfig = previewConfig.stats ? previewConfig.stats : {
          colors: true
        }; // eslint-disable-next-line no-console

        console.log(stats.toString(statsConfig));
      } else {
        _nodeLogger.logger.error(error.message);
      }
    });
  });
}

async function compilePreview(previewConfig, previewStartTime) {
  _nodeLogger.logger.info('=> Compiling preview..');

  return new Promise(function (resolve, reject) {
    (0, _webpack.default)(previewConfig).run(function (error, stats) {
      if (error || !stats || stats.hasErrors()) {
        _nodeLogger.logger.error('=> Failed to build the preview');

        process.exitCode = 1;

        if (error) {
          _nodeLogger.logger.error(error.message);

          return reject(error);
        }

        if (stats && (stats.hasErrors() || stats.hasWarnings())) {
          var _stats$toJson2 = stats.toJson(previewConfig.stats),
              warnings = _stats$toJson2.warnings,
              errors = _stats$toJson2.errors;

          errors.forEach(function (e) {
            return _nodeLogger.logger.error(e);
          });
          warnings.forEach(function (e) {
            return _nodeLogger.logger.error(e);
          });
          return reject(stats);
        }
      }

      _nodeLogger.logger.trace({
        message: '=> Preview built',
        time: process.hrtime(previewStartTime)
      });

      if (stats) {
        stats.toJson(previewConfig.stats).warnings.forEach(function (e) {
          return _nodeLogger.logger.warn(e);
        });
      }

      return resolve(stats);
    });
  });
}

async function copyAllStaticFiles(staticDir, outputDir) {
  if (staticDir && staticDir.length) {
    await Promise.all(staticDir.map(async function (dir) {
      var _dir$split$concat = dir.split(':').concat('/'),
          _dir$split$concat2 = _slicedToArray(_dir$split$concat, 2),
          currentStaticDir = _dir$split$concat2[0],
          staticEndpoint = _dir$split$concat2[1];

      var localStaticPath = _path.default.resolve(currentStaticDir);

      if (!(await _fsExtra.default.pathExists(localStaticPath))) {
        _nodeLogger.logger.error(`Error: no such directory to load static files: ${localStaticPath}`);

        process.exit(-1);
      }

      _shelljs.default.cp('-r', `${localStaticPath}/!(index.html)`, _path.default.join(outputDir, staticEndpoint));
    }));

    _nodeLogger.logger.info(`=> Copying static files from: ${staticDir.join(', ')}`);
  }
}

async function buildManager(configType, outputDir, configDir, options) {
  _nodeLogger.logger.info('=> Building manager..');

  var managerStartTime = process.hrtime();

  _nodeLogger.logger.info('=> Loading manager config..');

  var managerConfig = await (0, _managerConfig.default)(_objectSpread(_objectSpread({}, options), {}, {
    configType: configType,
    outputDir: outputDir,
    configDir: configDir,
    corePresets: [require.resolve('./manager/manager-preset.js')]
  }));

  if (options.debugWebpack) {
    (0, _logConfig.logConfig)('Manager webpack config', managerConfig);
  }

  return compileManager(managerConfig, managerStartTime);
}

async function buildPreview(configType, outputDir, packageJson, options) {
  var watch = options.watch,
      debugWebpack = options.debugWebpack;

  _nodeLogger.logger.info('=> Building preview..');

  var previewStartTime = process.hrtime();

  _nodeLogger.logger.info('=> Loading preview config..');

  var previewConfig = await (0, _config.default)(_objectSpread(_objectSpread({}, options), {}, {
    configType: configType,
    outputDir: outputDir,
    packageJson: packageJson,
    corePresets: [require.resolve('./preview/preview-preset.js')],
    overridePresets: [require.resolve('./preview/custom-webpack-preset.js')]
  }));

  if (debugWebpack) {
    (0, _logConfig.logConfig)('Preview webpack config', previewConfig);
  }

  if (watch) {
    return watchPreview(previewConfig);
  }

  return compilePreview(previewConfig, previewStartTime);
}

async function buildStaticStandalone(options) {
  var staticDir = options.staticDir,
      configDir = options.configDir,
      packageJson = options.packageJson;
  var configType = 'PRODUCTION';
  var outputDir = _path.default.isAbsolute(options.outputDir) ? options.outputDir : _path.default.join(process.cwd(), options.outputDir);

  var defaultFavIcon = require.resolve('./public/favicon.ico');

  _nodeLogger.logger.info(`=> Cleaning outputDir ${outputDir}`);

  if (outputDir === '/') throw new Error("Won't remove directory '/'. Check your outputDir!");
  await _fsExtra.default.emptyDir(outputDir);
  await (0, _cpy.default)(defaultFavIcon, outputDir);
  await copyAllStaticFiles(staticDir, outputDir);
  var prebuiltDir = await (0, _prebuiltManager.getPrebuiltDir)({
    configDir: configDir,
    options: options
  });

  if (prebuiltDir) {
    await (0, _cpy.default)('**', outputDir, {
      cwd: prebuiltDir,
      parents: true
    });
  } else {
    await buildManager(configType, outputDir, configDir, options);
  }

  if (options.managerOnly) {
    _nodeLogger.logger.info(`=> Not building preview`);
  } else {
    await buildPreview(configType, outputDir, packageJson, options);
  }

  _nodeLogger.logger.info(`=> Output directory: ${outputDir}`);
}

function buildStatic(_ref) {
  var packageJson = _ref.packageJson,
      loadOptions = _objectWithoutProperties(_ref, ["packageJson"]);

  var cliOptions = (0, _cli.getProdCli)(packageJson);
  return buildStaticStandalone(_objectSpread(_objectSpread(_objectSpread({}, cliOptions), loadOptions), {}, {
    packageJson: packageJson,
    configDir: loadOptions.configDir || cliOptions.configDir || './.storybook',
    outputDir: loadOptions.outputDir || cliOptions.outputDir || './storybook-static',
    ignorePreview: !!cliOptions.previewUrl,
    docsMode: !!cliOptions.docs
  })).catch(function (e) {
    _nodeLogger.logger.error(e);

    process.exit(1);
  });
}